@@ -1,25 +1,28 @@
 #!/usr/bin/env python
 # ex:ts=4:sw=4:sts=4:et
 # -*- tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*-
-"""class for handling configuration data files
+"""
+   class for handling configuration data files
 
    Reads a .conf file and obtains its metadata
 
-   Copyright (C) 2003, 2004  Chris Larson
-   Copyright (C) 2003, 2004  Phil Blundell
-   
-   This program is free software; you can redistribute it and/or modify it under
-   the terms of the GNU General Public License as published by the Free Software
-   Foundation; either version 2 of the License, or (at your option) any later
-   version.
-   
-   This program is distributed in the hope that it will be useful, but WITHOUT
-   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License along with
-   this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-   Place, Suite 330, Boston, MA 02111-1307 USA.""" 
+"""
+
+# Copyright (C) 2003, 2004  Chris Larson
+# Copyright (C) 2003, 2004  Phil Blundell
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 import re, bb.data, os, sys
 from bb.parse import ParseError
@@ -83,7 +86,7 @@ def obtain(fn, data):
     return localfn
 
 
-def include(oldfn, fn, data, error_out = False):
+def include(oldfn, fn, data, error_out):
     """
 
     error_out If True a ParseError will be reaised if the to be included
@@ -100,7 +103,7 @@ def include(oldfn, fn, data, error_out = False):
         ret = handle(fn, data, True)
     except IOError:
         if error_out:
-            raise ParseError("Could not include required file %(fn)s" % vars() )
+            raise ParseError("Could not %(error_out)s file %(fn)s" % vars() )
         bb.msg.debug(2, bb.msg.domain.Parsing, "CONF file '%s' not found" % fn)
 
 def handle(fn, data, include = 0):
@@ -117,14 +120,11 @@ def handle(fn, data, include = 0):
         oldfile = bb.data.getVar('FILE', data)
 
     fn = obtain(fn, data)
-    bbpath = []
     if not os.path.isabs(fn):
         f = None
-        vbbpath = bb.data.getVar("BBPATH", data)
-        if vbbpath:
-            bbpath += vbbpath.split(":")
-        for p in bbpath:
-            currname = os.path.join(bb.data.expand(p, data), fn)
+        bbpath = bb.data.getVar("BBPATH", data, 1) or []
+        for p in bbpath.split(":"):
+            currname = os.path.join(p, fn)
             if os.access(currname, os.R_OK):
                 f = open(currname, 'r')
                 abs_fn = currname
@@ -194,13 +194,13 @@ def feeder(lineno, s, fn, data):
     if m:
         s = bb.data.expand(m.group(1), data)
         bb.msg.debug(3, bb.msg.domain.Parsing, "CONF %s:%d: including %s" % (fn, lineno, s))
-        include(fn, s, data)
+        include(fn, s, data, False)
         return
 
     m = __require_regexp__.match(s)
     if m:
         s = bb.data.expand(m.group(1), data)
-        include(fn, s, data, True)
+        include(fn, s, data, "include required")
         return
 
     raise ParseError("%s:%d: unparsed line: '%s'" % (fn, lineno, s));
